#' install python dependencies for the floodam.building hydraulic model
#' 
#' @details manage_python() manage the installation of the floodam.building's 
#'  python dependencies. it creates a new virtual environment so the user's 
#'  current python installation is not modified 
#' 
#' @param envname character; name of the python's virtual environment. 
#'      Default to "floodam"
#' @param packages character; names of the packages to install. 
#'      Default to "numpy" and "pandas"
#' @param force logical; force the creation of the virtual environment and the 
#'      reinstallation of dependencies. Default to FALSE 
#' 
#' @param remove logical; remove the virtual environment. Default to FALSE 
#' 
#' @export 

manage_python = function(
    envname = "floodam",
    packages = c("numpy", "pandas"),
    force = FALSE,
    remove = FALSE
) {

    # check whether python is installed and whether the version installed is
    # higher than v3. If it is not installed or the version installed is
    # inferior to v3 the latest version of python 3.10 is installed
    py_installed_version = getOption("floodam_building_py")
    if (!compatible_python()) {
		stop("Python 3 not detected: hydraulic tools not available")
	}

    # check whether the python virtual environment exists
    venv_exist = reticulate::virtualenv_exists(envname)

    if (isTRUE(venv_exist) && isTRUE(remove)) {
        message(sprintf("Removing virtual environment '%s'\n", envname))
        reticulate::virtualenv_remove(
            envname = envname,
            confirm = FALSE    
        )
        return(invisible())
    }
    # if virtual environment exists check if dependencies are installed
    if (isTRUE(venv_exist) && isFALSE(force)) {
        pkg_inst = all(
            packages %in% reticulate::py_list_packages(envname)[["package"]]
        )
        if (isTRUE(pkg_inst)) {
            message(
                sprintf(
                    "Virtual environment '%s' found: nothing to do", 
                    envname
                )
            )
        } else {
            message(sprintf("Virtual environment '%s' found\n", envname))
            message("Dependencies not satisfied. 
                    Proceeding to install additional packages")
            reticulate::virtualenv_install(
                envname = envname, 
                packages = packages,
                python_version = py_installed_version
            )
        }
        return(invisible())
    }

    # remove the "floodam.building" virtual environment if force = TRUE and 
    # reinstall
    if (isTRUE(venv_exist) && isTRUE(force)) {    
        reticulate::virtualenv_create(
            envname = envname, 
            python = py_installed_version, 
            packages = packages,
            force = force
        )
        return(invisible())
    }

    # if virtual environment does not exist or if the dependencies are not 
    # installed, create virtual environment and install dependencies
    if (isFALSE(venv_exist) && isFALSE(remove)) {
        reticulate::virtualenv_create(
            envname = envname, 
            python = py_installed_version, 
            packages = packages
        )
        return(invisible())
    }
}

compatible_python = function(x = options("floodam_building_py")) {
    ! (is.null(x) || x < 3)
}